from cgi import parse_header

from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseServerError
from django.utils.decorators import method_decorator
from django.views.decorators.clickjacking import xframe_options_exempt
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from .receive import Receiver


# TODO: check/set server header?
# TODO: remove Vary: Accept-Language, Cookie
# TODO: remove Content-Language: en

class ReceiveView(Receiver, View):

    @method_decorator(csrf_exempt)
    @method_decorator(xframe_options_exempt)
    def dispatch(self, *args, **kwargs):
        return super(ReceiveView, self).dispatch(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        content_type, params = parse_header(request.META.get("CONTENT_TYPE"))
        if content_type.lower() != "application/json":
            resp = HttpResponse(status=415)
            resp["Content-Length"] = "0"
            return resp
        try:
            data = self.parse_json(request.body, params.get("charset", "UTF-8"))
        except:
            resp = HttpResponseBadRequest()
            resp["Warning"] = '199 - "JSON parsing error"'
            resp["Content-Length"] = "0"
            return resp
        try:
            self.handle_input(data)
        except:
            resp = HttpResponseServerError()
            resp["Warning"] = '199 - "Input handling error"'
            resp["Content-Length"] = "0"
            return resp
        return HttpResponse(status=204)


ReceiveView.put = ReceiveView.post
